<?php

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Report\ReportService;
use Teknisa\MAN\Services;
use Zeedhi\Framework\DTO;
use Teknisa\MAN\Utilities;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
class RelMapaResumoECF{
    
    const PARAM_P_TELA = 'P_';
    const PARAM_PT_TELA = 'P_T';
    protected $relatorioService;
    protected $relMapaService;
    protected $utilService;
    protected $birtReport;
    protected $nameReport = 'mapaResumoECF';
    protected $insNrMapaResECF;
    protected $retirarParams = array('NMLOJA','NMFILIAL','changed','DTENTRVENDA','CDLOJACAIXA','NMCAIXA','NMREGIAO','CDREGIAO','CDFILIAL','CDCAIXA','CDLOJA');
    protected $inserirParams = array(
        'NRECF'=>'3',
        'NRSEQUENCIA'=> null,
        'ORIGVENDA' => 'I'
    );
    protected $checkParams = array('CDREGIAO','CDFILIAL','CDLOJA','CDCAIXA');
    
    public function __construct (Utilities\Util $utilService,ReportService $reportService, Services\Relatorio $relatorioService, BirtReport $birtReport,
        Services\RelMapaResumoECF $relMapaService, Services\InsNrMapaResECF $insNrMapaResEC) {
        $this->reportService    = $reportService;
        $this->relatorioService = $relatorioService;
        $this->utilService      = $utilService;
        $this->birtReport       = $birtReport;
        $this->relMapaService   = $relMapaService;
        $this->insNrMapaResEC   = $insNrMapaResEC;
    }
    
    public function geraRelatorio(DTO\Request\Row $request, DTO\Response $response) {
        $conditions = $this->utilService->getParamsAsArray($request);
        $conditions = $this->buildConditions($conditions);
        $rows = $this->relMapaService->checkAdqMapaResumo($conditions);
        if(!isset($rows['MSG'])){
            $this->insNrMapaResEC->stNrMapaResECF($conditions,$rows);
            $conditions['__report_name'] = $this->getNameReport(isset($conditions['TIPORELATORIO'])?$conditions['TIPORELATORIO']:"SP");
            $params = $this->buildParamsAsArray($conditions);
            $this->getRelatorio($params, $response);
        }else{
            $response->addMessage(new Message($rows['MSG'], Message::TYPE_MESSAGE));
        }
    }
    
    public function getRelatorio ($params , DTO\Response $response){
         try {
            $birtURL = $this->reportService->createRemoteReport('BIRT', $params['ReportName'], $params['ParametersRow']);
            $response->addDataset(new DataSet('URL',array('url'=>$birtURL)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage() , 500));
        }
     }
     
    
    private function getNameReport($name){
        if($name==''){
            return $this->nameReport;
        }
        return $this->nameReport.$name;
    }
    
    private function parameterMapping($conditions){
        $parameterMapping = array();
        foreach ($conditions as $key=>$value) {
            $parameterMapping[$key] = $key;
        }
        unset($parameterMapping['__report_name']);
        return $parameterMapping;
    }
    
    private function buildConditions($conditions){
        $conditions = $this->buildParamsCaixa($conditions);
        $conditions = $this->buildParams($conditions);
        $conditions = $this->buildDataVenda($conditions);
        $conditions = $this->retirarParams($conditions);
        $conditions = $this->inserirParams($conditions);
        return $conditions;
    }
    
    private function buildParamsCaixa ($conditions){
        if(isset($conditions['CDCAIXA'])){
            $conditions['CDCAIXA'] =  str_replace('_','',$conditions['CDCAIXA']);    
        }
        return $conditions;
    }
    
    private function buildParams ($conditions){
        foreach ($this->checkParams as $value) {
            if(!array_key_exists($value, $conditions) || (is_array($conditions[$value]) && !count($conditions[$value]))) {
                $conditions[$this->insereP($value)] = '';
                $conditions[$this->insereP_T($value)] = 'T';
            }else{
                $conditions[$this->insereP($value)] = is_array($conditions[$value]) ? implode('_', $conditions[$value]) : '';
                $conditions[$this->insereP_T($value)] = '';
            }
        }
        return $conditions;
     }
     
    private function insereP ($text){
        return self::PARAM_P_TELA.$text;
    }
    
    private function insereP_T ($text){
        return self::PARAM_PT_TELA.$text;
    }
    
    private function buildDataVenda ($conditions){
        if(array_key_exists('DTENTRVENDA', $conditions)) {
            $conditions[$this->insereP('DTVENDAINI')] = $conditions['DTENTRVENDA']['start'];
            $conditions[$this->insereP('DTVENDAFIN')] = $conditions['DTENTRVENDA']['end'];
        }
        return $conditions;
    }
    
    private function retirarParams($conditions){
        foreach ($this->retirarParams as $value) {
            if(isset($conditions[$value])){
                unset($conditions[$value]);
            }
        }
        return $conditions;
    }
    
    private function inserirParams($conditions){
        foreach ($this->inserirParams as $key =>  $value) {
            $conditions[$this->insereP($key)] = $value;
        }
        return $conditions;
    }
    
    private function buildParamsAsArray($conditions){
        $params = array(
            "TIPORELATORIO"     => $conditions['TIPORELATORIO'],
            "P_AGRUPA"          => $conditions['P_AGRUPA'],
            "NRORG"             => $conditions['NRORG'],
            "CDFILIAL_CURRENT"  => $conditions['CDFILIAL_CURRENT'],
            "P_CDREGIAO"        => $conditions['P_CDREGIAO'],
            "P_TCDREGIAO"       => $conditions['P_TCDREGIAO'],
            "P_CDFILIAL"        => $conditions['P_CDFILIAL'],
            "P_TCDFILIAL"       => $conditions['P_TCDFILIAL'],
            "P_CDLOJA"          => $conditions['P_CDLOJA'],
            "P_TCDLOJA"         => $conditions['P_TCDLOJA'],
            "P_CDCAIXA"         => $conditions['P_CDCAIXA'],
            "P_TCDCAIXA"        => $conditions['P_TCDCAIXA'],
            "P_DTVENDAINI"      => $conditions['P_DTVENDAINI'],
            "P_DTVENDAFIN"      => $conditions['P_DTVENDAFIN'],
            "P_NRECF"           => $conditions['P_NRECF'],
            "P_NRSEQUENCIA"     => $conditions['P_NRSEQUENCIA'],
            "P_ORIGVENDA"       => $conditions['P_ORIGVENDA']
        );
        $params = array(
            'ParametersRow' => $params,
            'ReportName'    => $conditions['__report_name']
        );
        return $params;
    }
}